/*
  You are referred to the LHAPDF library
      http://projects.hepforge.org/lhapdf/
  for detailed explainations about the functions used here.
*/

#include <string>
//#include "LHAPDF/LHAPDF.h"  // PDF library LHAPDF
#include "LHAPDF_interface.hh"
#include "QCD_EW.hh"

using namespace std;



bool PDF::initialized = false;

int PDF::check_initialization() {
  if(!PDF::initialized) {
    cerr << " >>>>> PDF set not initialized !    -->  Exit" << endl;
    exit(-19);
  }
  return 0;
}

// initialize a pdf set
int PDF::pdf_init(string SET_NAME, int SUBSET, verbosity v) {
  setVerbosity((LHAPDF::Verbosity)v);
  LHAPDF::initPDFSet(SET_NAME, LHAPDF::LHGRID, SUBSET);
  cout << endl << " >>>>> Using PDFs:  " << SET_NAME << " ,  subset: " << SUBSET << endl << endl;
  initialized = true;
  return 0;
}

// Extrapolate beyond grid edges.
void PDF::extrapolate(bool extrapolate) {
  LHAPDF::extrapolate(extrapolate);
}

double PDF::alphas_PDF(double Q) {
  return LHAPDF::alphasPDF(Q);
}


// call pdfs in a standard notation, after the above initialization:
double PDF::pdf (int pdf_number, double x, double mu) { return LHAPDF::xfx(x,mu, pdf_number)/x; };

double PDF::d   (double x, double mu) { return LHAPDF::xfx(x,mu, 1)/x; };
double PDF::dbar(double x, double mu) { return LHAPDF::xfx(x,mu,-1)/x; };

double PDF::u   (double x, double mu) { return LHAPDF::xfx(x,mu, 2)/x; };
double PDF::ubar(double x, double mu) { return LHAPDF::xfx(x,mu,-2)/x; };

double PDF::s   (double x, double mu) { return LHAPDF::xfx(x,mu, 3)/x; };
double PDF::str (double x, double mu) { return LHAPDF::xfx(x,mu, 3)/x; };
double PDF::sbar(double x, double mu) { return LHAPDF::xfx(x,mu,-3)/x; };

double PDF::c   (double x, double mu) { return LHAPDF::xfx(x,mu, 4)/x; };
double PDF::chm (double x, double mu) { return LHAPDF::xfx(x,mu, 4)/x; };
double PDF::cbar(double x, double mu) { return LHAPDF::xfx(x,mu,-4)/x; };

double PDF::b   (double x, double mu) { return LHAPDF::xfx(x,mu, 5)/x; };
double PDF::bot (double x, double mu) { return LHAPDF::xfx(x,mu, 5)/x; };
double PDF::bbar(double x, double mu) { return LHAPDF::xfx(x,mu,-5)/x; };

double PDF::g   (double x, double mu) { return LHAPDF::xfx(x,mu, 0)/x; };


// same as before, but calls x*f(x,Q)
double PDF::xpdf (int pdf_number, double x, double mu) { return LHAPDF::xfx(x,mu, pdf_number); };

double PDF::xd   (double x, double mu) { return LHAPDF::xfx(x,mu, 1); };
double PDF::xdbar(double x, double mu) { return LHAPDF::xfx(x,mu,-1); };

double PDF::xu   (double x, double mu) { return LHAPDF::xfx(x,mu, 2); };
double PDF::xubar(double x, double mu) { return LHAPDF::xfx(x,mu,-2); };

double PDF::xs   (double x, double mu) { return LHAPDF::xfx(x,mu, 3); };
double PDF::xstr (double x, double mu) { return LHAPDF::xfx(x,mu, 3); };
double PDF::xsbar(double x, double mu) { return LHAPDF::xfx(x,mu,-3); };

double PDF::xc   (double x, double mu) { return LHAPDF::xfx(x,mu, 4); };
double PDF::xchm (double x, double mu) { return LHAPDF::xfx(x,mu, 4); };
double PDF::xcbar(double x, double mu) { return LHAPDF::xfx(x,mu,-4); };

double PDF::xb   (double x, double mu) { return LHAPDF::xfx(x,mu, 5); };
double PDF::xbot (double x, double mu) { return LHAPDF::xfx(x,mu, 5); };
double PDF::xbbar(double x, double mu) { return LHAPDF::xfx(x,mu,-5); };

double PDF::xg   (double x, double mu) { return LHAPDF::xfx(x,mu, 0); };


void PDF::PrintPDFInfo(int SUBSET) {
  LHAPDF::getDescription();
  cout << " Parameters:" << endl;
  cout << "  alpha_s(m_Z) = " << LHAPDF::alphasPDF(MASS::mass_Z) << endl;
  cout << "  qcdlam4      = " << LHAPDF::getLam4(SUBSET) << endl;
  cout << "  qcdlam5      = " << LHAPDF::getLam5(SUBSET) << endl;
  cout << "  orderPDF     = " << LHAPDF::getOrderPDF() << endl;
  cout << "  xmin         = " << LHAPDF::getXmin(SUBSET) << endl;
  cout << "  xmax         = " << LHAPDF::getXmax(SUBSET) << endl;
  cout << "  q2min        = " << LHAPDF::getQ2min(SUBSET) << endl;
  cout << "  q2max        = " << LHAPDF::getQ2max(SUBSET) << endl;
  cout << "  orderalfas   = " << LHAPDF::getOrderAlphaS() << endl;
  cout << "  num flav     = " << LHAPDF::getNf() << endl;
  cout << "  number       = " << LHAPDF::numberPDF() << endl << endl;
}
