/*
  This header contains the link to the LHAPDF libraries.
  Every function defined there may be called using

    LHAPDF::function_defined_in_lhapdf(....)

  The functions defined here are called using

    PDF::function_defined_here(....)

  and are essentially aliases redefined here for convenience.
  You are referred to the LHAPDF library
       http://projects.hepforge.org/lhapdf/
  for detailed explainations.
*/

#ifndef PDF_H
#define PDF_H

#include <string>
#include <iostream>
#include "LHAPDF/LHAPDF.h"  // PDF library LHAPDF

enum verbosity { SILENT=0, LOWKEY=1, DEFAULT=2 };


namespace PDF {

  extern bool initialized;
  extern int check_initialization();

  // initialize a pdf.
  extern int pdf_init(std::string SET_NAME, int SUBSET=0, verbosity v=DEFAULT);
  extern void extrapolate(bool extrapolate=true);
  extern double alphas_PDF(double Q);
  extern void PrintPDFInfo(int SUBSET=0);
  
  // call pdfs in a standard notation, after the above initialization:
  extern double pdf (int pdf_number, double x, double mu);
  extern double u   (double x, double mu);
  extern double ubar(double x, double mu);
  extern double d   (double x, double mu);
  extern double dbar(double x, double mu);
  extern double s   (double x, double mu);
  extern double str (double x, double mu);
  extern double sbar(double x, double mu);
  extern double c   (double x, double mu);
  extern double chm (double x, double mu);
  extern double cbar(double x, double mu);
  extern double b   (double x, double mu);
  extern double bot (double x, double mu);
  extern double bbar(double x, double mu);
  extern double g   (double x, double mu);

// same as before, but calls x*f(x,Q)
  extern double xpdf (int pdf_number, double x, double mu);
  extern double xu   (double x, double mu);
  extern double xubar(double x, double mu);
  extern double xd   (double x, double mu);
  extern double xdbar(double x, double mu);
  extern double xs   (double x, double mu);
  extern double xstr (double x, double mu);
  extern double xsbar(double x, double mu);
  extern double xc   (double x, double mu);
  extern double xchm (double x, double mu);
  extern double xcbar(double x, double mu);
  extern double xb   (double x, double mu);
  extern double xbot (double x, double mu);
  extern double xbbar(double x, double mu);
  extern double xg   (double x, double mu);
  // Alekhin writes an alpha_s output too:
  //double Alekhin_alpha_s(double Q);

};

# endif    /*  PDF_H  */
